﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Configuration;
using System.Data.SqlClient;
using System.Data.EntityClient;

namespace mobx.mobi.Core
{
    public class CoreCommon
    {
        public static T GetClassAttribute<T>(Type attrClass) where T : class
        {
            object obj = attrClass.GetCustomAttributes(true).Where(a=>a is T).FirstOrDefault();
            if (obj != null)
            {
                return (T)obj;
            }
            else
            {
                return null;
            }
        }
        public static string GetConnectionString()
        {
            if (ConfigurationManager.ConnectionStrings["ConnectionString"] != null &&
                !string.IsNullOrEmpty(ConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString))
            {
                return ConfigurationManager.ConnectionStrings["ConnectionString"].ConnectionString;
            }
            else
                return null;
        }
        public static string ConvertEFConnStringToSQLProviderConnString(string entityConnString)
        {
            var efBuilder = new EntityConnectionStringBuilder(entityConnString);
            var sqlBuilder = new SqlConnectionStringBuilder(efBuilder.ProviderConnectionString);
            sqlBuilder.ApplicationName = null;
            return sqlBuilder.ConnectionString;
        }

        public static string CreateEFConnectionString(bool trustedConnection, string serverName, string databaseName, string userName, string password, int timeout = 0)
        {
            string sqlProviderConnectionString = CreateSqlProviderConnectionString(trustedConnection, serverName, databaseName, userName, password, timeout);
            return CreateEFConnectionString(sqlProviderConnectionString);
        }
        public static string CreateEFConnectionString(string sqlProviderConnectionString)
        {
            // Initialize the EntityConnectionStringBuilder.
            EntityConnectionStringBuilder entityBuilder = new EntityConnectionStringBuilder();
            //Set the provider name.
            entityBuilder.Provider = "System.Data.SqlClient";

            // Set the provider-specific connection string.
            entityBuilder.ProviderConnectionString = sqlProviderConnectionString;

            // Set the Metadata location.
            entityBuilder.Metadata = @"res://*/DataModel.csdl|res://*/DataModel.ssdl|res://*/DataModel.msl";
            string efConnectionString = entityBuilder.ConnectionString;
            return efConnectionString;
        }
        /// <summary>
        /// Create contents of connection strings used by the SqlConnection class
        /// </summary>
        /// <param name="trustedConnection">Avalue that indicates whether User ID and Password are specified in the connection (when false) or whether the current Windows account credentials are used for authentication (when true)</param>
        /// <param name="serverName">The name or network address of the instance of SQL Server to connect to</param>
        /// <param name="databaseName">The name of the database associated with the connection</param>
        /// <param name="userName">The user ID to be used when connecting to SQL Server</param>
        /// <param name="password">The password for the SQL Server account</param>
        /// <param name="timeout">The connection timeout</param>
        /// <returns>Connection string</returns>
        public static string CreateSqlProviderConnectionString(bool trustedConnection, string serverName, string databaseName, string userName, string password, int timeout = 0)
        {
            var builder = new SqlConnectionStringBuilder();
            builder.IntegratedSecurity = trustedConnection;
            builder.DataSource = serverName;
            builder.InitialCatalog = databaseName;
            if (!trustedConnection)
            {
                builder.UserID = userName;
                builder.Password = password;
            }
            builder.PersistSecurityInfo = false;
            builder.MultipleActiveResultSets = true;
            if (timeout > 0)
            {
                builder.ConnectTimeout = timeout;
            }
            builder.ApplicationName = "EntityFramework";
            string sqlProviderConnectionString = builder.ConnectionString;
            return sqlProviderConnectionString;
        }
    }
}
