﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace mobx.mobi.DAL.Managers
{
    public class MenuManager
    {
        public MenuManager()
        {            
        }

        public List<MbMenu> GetSiteMenu()
        {
            List<MbMenu> lst = Singleton.DB.MbMenus.OrderBy(m=>m.SortOrder).ToList();
            return lst;
        }
        public MbMenu GetMenu(int menuId)
        {
            MbMenu menu = Singleton.DB.MbMenus.Where(m => m.MenuId == menuId).FirstOrDefault();
            return menu;
        }
        public void DeleteMenu(int menuId)
        {
            MbMenu menu = Singleton.DB.MbMenus.Where(m => m.MenuId == menuId).FirstOrDefault();
            if (menu != null)
            {
                Singleton.DB.MbMenus.DeleteObject(menu);
                Singleton.DB.SaveChanges();
            }
        }
        public void ShiftUp(int menuId)
        {
            MbMenu menu = Singleton.DB.MbMenus.Where(m => m.MenuId == menuId).FirstOrDefault();
            if (menu != null &&
                menu.SortOrder > 0)
            {
                MbMenu oneUp = Singleton.DB.MbMenus.Where(m => m.SortOrder == menu.SortOrder-1).FirstOrDefault();
                if (oneUp != null)
                {
                    oneUp.SortOrder++;
                    menu.SortOrder--;
                    Singleton.DB.SaveChanges();
                }
            }
        }
        public void ShiftDown(int menuId)
        {
            MbMenu menu = Singleton.DB.MbMenus.Where(m => m.MenuId == menuId).FirstOrDefault();
            int maxSortOrder = GetMaxSortOrder();
            if (menu != null &&
                menu.SortOrder < maxSortOrder)
            {
                MbMenu oneDown = Singleton.DB.MbMenus.Where(m => m.SortOrder == menu.SortOrder + 1).FirstOrDefault();
                if (oneDown != null)
                {
                    oneDown.SortOrder--;
                    menu.SortOrder++;
                    Singleton.DB.SaveChanges();
                }
            }
        }
        public MbMenu AddMenu(int? linkToPageId, string linkText, string linkUrl)
        {
            MbMenu menu = null;
            try
            {
                if ((linkToPageId.HasValue && linkToPageId.Value > 0)
                    ||
                    (!string.IsNullOrEmpty(linkText) && !string.IsNullOrEmpty(linkUrl)))
                {
                    int sortOrder = GetMaxSortOrder();
                    sortOrder++;

                    menu = new MbMenu();
                    menu.SortOrder = sortOrder;
                    if (linkToPageId.HasValue && linkToPageId.Value > 0)
                    {
                        menu.LinkToPageId = linkToPageId;
                        menu.LinkText = null;
                        menu.LinkUrl = null;
                    }
                    else
                    {
                        menu.LinkToPageId = null;
                        menu.LinkText = linkText;
                        menu.LinkUrl = linkUrl;
                    }
                    Singleton.DB.MbMenus.AddObject(menu);
                    Singleton.DB.SaveChanges();
                }
            }
            catch
            {
                menu = null;
            }
            return menu;
        }
        private int GetMaxSortOrder()
        {
            int sortOrder = -1;
            var menus = Singleton.DB.MbMenus.OrderByDescending(m => m.SortOrder);
            if (menus != null &&
                menus.Any())
            {
                sortOrder = menus.Select(m => m.SortOrder).FirstOrDefault();
            }
            return sortOrder;
        }
    }
}
