﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;

namespace mobx.mobi.DAL.Managers
{
    public class PageManager
    {
        public PageManager()
        {
        }
        public MbPage GetDefaultPage()
        {           
            MbPage defaultPage = null;
            SettingManager setMan = new SettingManager();
            int pageId = setMan.GetDefaultPageId();
            if (pageId > 0)
            {
                defaultPage = Singleton.DB.MbPages.Where(p => p.PageId == pageId)
                                                  .FirstOrDefault();
            }
            
            if (defaultPage == null)
            {
                defaultPage = Singleton.DB.MbPages.Where(p => p.Published)
                                                  .OrderBy(p => p.PageId)
                                                  .FirstOrDefault();
            }
            return defaultPage;
        }

        public List<MbPage> GetPages(bool showOnlyPublished)
        {
            List<MbPage> sitePages = null;
            if (showOnlyPublished)
            {
                sitePages = Singleton.DB.MbPages.Where(p=>p.Published == showOnlyPublished).ToList();
            }
            else
            {
                sitePages = Singleton.DB.MbPages.ToList();
            }
            return sitePages;
        }

        public MbPage GetPage(int pageId)
        {
            MbPage page = Singleton.DB.MbPages.Where(p => p.PageId == pageId).FirstOrDefault();
            return page;
        }

        public MbPage GetPage(string pageAlias)
        {
            MbPage page = Singleton.DB.MbPages.Where(p => p.Alias == pageAlias).FirstOrDefault();
            return page;
        }

        public void DeletePage(int pageId, out string errorMessage)
        {
            try
            {
                MbPage mbpage = GetPage(pageId);
                if (mbpage != null)
                {
                    Singleton.DB.MbPages.DeleteObject(mbpage);
                    Singleton.DB.SaveChanges();
                }
                errorMessage = null;
            }
            catch (Exception ex)
            {
                errorMessage = ex.Message;
            }
        }
        public bool IsPageAliasValid(int pageId, string pageAlias)
        {
            MbPage foundPage = null;
            return IsPageAliasValid(pageId, pageAlias, out foundPage);
        }

        public bool IsPageAliasValid(int pageId, string pageAlias, out MbPage foundPage)
        {
            bool valid = true;
            foundPage = GetPage(pageAlias);
            if (pageId > 0 &&
                foundPage != null &&
                foundPage.PageId != pageId)
            {
                valid = false;
                //we want to rename our page to a name that already exists and it's not me
            }
            else if (pageId <= 0 &&
                        foundPage != null)
            {
                valid = false;
            }
            else if (pageId > 0 &&
                    foundPage == null)
            {
                //this can happen when alias of existing page changes, then get page by pageId
                foundPage = GetPage(pageId);
            }

            if (!string.IsNullOrEmpty(pageAlias) &&
                pageAlias.ToLower() == "default")
            {
                valid = false;
                foundPage = null;
            }
            return valid;
        }

        public MbPage UpsertPage(MbPage mbPage, out string errorMessage)
        {
            try
            {           
                if (mbPage != null)
                {
                    MbPage foundPage = null;
                    bool aliasValid = IsPageAliasValid(mbPage.PageId, mbPage.Alias, out foundPage);
                    if (!aliasValid)
                    {
                        //a page with the same alias already exists
                        errorMessage = "A page with the same URL Name already exists";
                    }
                    else
                    {
                        if (mbPage.PageId == 0)
                        {
                            Singleton.DB.MbPages.AddObject(mbPage);
                            Singleton.DB.SaveChanges();
                        }
                        else
                        {
                            foundPage.Alias = mbPage.Alias;
                            foundPage.MetaDescription = mbPage.MetaDescription;
                            foundPage.MetaKeywords = mbPage.MetaKeywords;
                            foundPage.PageContent = mbPage.PageContent;
                            //foundPage.PageId = mbPage.PageId;
                            foundPage.PageType = mbPage.PageType;
                            foundPage.Published = mbPage.Published;
                            //foundPage.SiteId = mbPage.SiteId;
                            foundPage.Title = mbPage.Title;
                            foundPage.MasterPage = mbPage.MasterPage;
                            //Singleton.DB.MbPages.Attach(mbPage);
                            //Singleton.DB.ObjectStateManager.ChangeObjectState(mbPage, EntityState.Modified);
                            Singleton.DB.SaveChanges();
                            mbPage.PageId = foundPage.PageId;
                        }
                    }
                }
                errorMessage = null;
            }
            catch (Exception ex)
            {
                errorMessage = ex.ToString();
            }
            return mbPage;
        }
    }
}
