﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace mobx.mobi.DAL.Managers
{
    public class RoleManager
    {
        public RoleManager()
        {
        }
        /// <summary>
        /// Gets customer roles by customer identifier
        /// </summary>
        /// <param name="userId">Customer identifier</param>
        /// <param name="showHidden">A value indicating whether to show hidden records</param>
        /// <returns>Customer role collection</returns>
        public List<MbUserRole> GetUserRolesByUserId(int userId, bool showHidden)
        {
            if (userId == 0)
                return new List<MbUserRole>();

            var query = Singleton.DB.MbUsers.Where(u => u.Active || showHidden)
                .Where(u => !u.Deleted)
                .Where(u => u.UserId == userId)
                .Join(Singleton.DB.MbUser_UserRole_Mapping, u => u.UserId, m => m.UserId, (u, m) => m)
                .Join(Singleton.DB.MbUserRoles, m => m.UserRoleId, r => r.UserRoleId, (m, r) => r)
                .OrderBy(r => r.Name)
                .Select(r => r)
                .ToList();

            var userRoles = query.ToList();
            return userRoles;
        }

        public List<MbUser> GetUsersInRole(string roleName, bool showHidden)
        {
            if (string.IsNullOrEmpty(roleName))
                return null;

            var query = Singleton.DB.MbUserRoles.Where(r=>r.Active)
                                                .Where(r=>!r.Deleted)
                                                .Where(r=>r.Name == roleName)
                                                .Join(Singleton.DB.MbUser_UserRole_Mapping, r=>r.UserRoleId, m=>m.UserRoleId, (r,m)=>m)
                                                .Join(Singleton.DB.MbUsers, m=>m.UserId, u=>u.UserId, (m,u)=>u)
                                                .Where(u=>!u.Deleted)
                                                .Where(u=>u.Active || showHidden)
                                                .Select(u=>u);

            var users = query.ToList();
            return users;
        }

        public void AddUserToRoles(int userId, List<int> roleIds)
        {
            if (userId > 0 && 
                roleIds != null && 
                roleIds.Any())
            {
                for (int i = 0; i < roleIds.Count; i++)
                {
                    MbUser_UserRole_Mapping mapping = new MbUser_UserRole_Mapping
                    {
                        UserId = userId,
                        UserRoleId = roleIds[i]
                    };
                    Singleton.DB.MbUser_UserRole_Mapping.AddObject(mapping);
                }
                Singleton.DB.SaveChanges();
            }
        }

        public void RemoveRolesFromUser(int userId, List<int> roleIds)
        {
            if (userId > 0 &&
                roleIds != null &&
                roleIds.Any())
            {
                for (int i = 0; i < roleIds.Count; i++)
                {
                    var roleMapping = GetRoleMapping(userId, roleIds[i]);
                    if (roleMapping != null)
                    {
                        Singleton.DB.MbUser_UserRole_Mapping.DeleteObject(roleMapping);
                    }
                }
                Singleton.DB.SaveChanges();
            }
        }

        public MbUser_UserRole_Mapping GetRoleMapping(int userId, int roleId)
        {
            MbUser_UserRole_Mapping mapping = null;
            if (roleId > 0 && userId >0)
            {
                mapping = Singleton.DB.MbUser_UserRole_Mapping.Where(m => m.UserRoleId == roleId)
                                                           .Where(m=>m.UserId == userId)
                                                           .FirstOrDefault();
            }
            return mapping;
        }

        public MbUserRole GetRole(int roleId)
        {
            MbUserRole role = null;
            if (roleId > 0)
            {
                role = Singleton.DB.MbUserRoles.Where(r => r.UserRoleId == roleId)
                                               .Where(r => r.Active)
                                               .Where(r => !r.Deleted)
                                             .FirstOrDefault();
            }
            return role;
        }

        public MbUserRole GetRoleByName(string roleName)
        {
            MbUserRole role = null;
            if (!string.IsNullOrEmpty(roleName))
            {
                role = Singleton.DB.MbUserRoles.Where(r => r.Name == roleName)
                                               .Where(r => r.Active)
                                               .Where(r => !r.Deleted)
                                             .FirstOrDefault();
            }
            return role;
        }

        public MbUserRole AddRole(string roleName)
        {
            MbUserRole role = new MbUserRole
            {
                Active = true,
                Deleted = false,
                Name = roleName,
                PackageId = 0
                // UserRoleId 
            };
            Singleton.DB.MbUserRoles.AddObject(role);
            Singleton.DB.SaveChanges();
            return role;
        }

        public bool DeleteRole(string roleName)
        {
            MbUserRole role = GetRoleByName(roleName);
            if (role != null)
            {
                Singleton.DB.MbUserRoles.DeleteObject(role);
                Singleton.DB.SaveChanges();
                return true;
            }
            return false;
        }

        public List<MbUserRole> GetAllRoles()
        {
            List<MbUserRole> roles = Singleton.DB.MbUserRoles.Select(x=>x).ToList();
            return roles;
        }
    }
}
