﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using mobx.mobi.Areas.Admin.Models;
using mobx.mobi.DAL;
using mobx.mobi.DAL.Managers;
using mobx.mobi.Models;
using System.Web.Caching;

namespace mobx.mobi.Areas.Admin.Controllers
{
    public class AccountController : Controller
    {

        //
        // GET: /Admin/Account/LogOn

        public ActionResult Login()
        {
            string returnUrl = Request.QueryString["ReturnUrl"];
            //check if already logged in
            if (Request.IsAuthenticated)
            {
                if (!string.IsNullOrEmpty(returnUrl) && Url.IsLocalUrl(returnUrl) && returnUrl.Length > 1 && returnUrl.StartsWith("/")
                    && !returnUrl.StartsWith("//") && !returnUrl.StartsWith("/\\"))
                {
                    return Redirect(returnUrl);
                }
                else
                {
                    return RedirectToAction("Index", "Default");
                }
            }
            return View();
        }

        /// <summary>
        /// You must check the user password, make sure the user is active, and not deleted before using this method
        /// </summary>
        /// <returns>ActionResult</returns>
        private ActionResult LoginUser(string email, bool rememberMe, string returnUrl, bool showErrors)
        {
            bool siteNotOwned = false;
            if (Roles.IsUserInRole(email, "GlobalAdmin"))
            {
                UserManager userManager = new UserManager();
                MbUser mbUser = userManager.GetUserByEmail(email);
                if (mbUser != null)
                {
                    FormsAuthentication.SetAuthCookie(email, rememberMe);
                    if (!string.IsNullOrEmpty(returnUrl) && Url.IsLocalUrl(returnUrl) && returnUrl.Length > 1 && returnUrl.StartsWith("/")
                        && !returnUrl.StartsWith("//") && !returnUrl.StartsWith("/\\"))
                    {
                        return Redirect(returnUrl);
                    }
                    else
                    {
                        return Redirect("~/Admin/");
                    }
                }
            }

            if (showErrors)
            {
                // If we got this far, something failed, redisplay form with error message
                if (siteNotOwned)
                    ModelState.AddModelError("", "You are not the owner of this site");
                else
                    ModelState.AddModelError("", "Wrong email or password");
            }
            return View();
        }

        //
        // POST: /Account/LogOn

        [HttpPost]
        public ActionResult Login(LoginModel model, string returnUrl)
        {
            if (ModelState.IsValid &&
                Membership.ValidateUser(model.Email, model.Password))
            {
                return LoginUser(model.Email, model.RememberMe, returnUrl, true);
            }

            // If we got this far, something failed, redisplay form with error message
            ModelState.AddModelError("", "Wrong email or password");
            return View(model);
        }

        public ActionResult ResetPassword()
        {
            return View();            
        }

        [HttpPost]
        public ActionResult ResetPassword(ResetPasswordModel model)
        {
            if (ModelState.IsValid)
            {
                var user = Membership.GetUser(model.Email, false);
                if (user != null)
                {
                    //check if this user is owner of the current site
                    UserManager userManager = new UserManager();
                    MbUser mbUser = userManager.GetUserByEmail(model.Email);
                    if (mbUser != null)
                    {
                        string newPassword = user.ResetPassword();
                        if (!string.IsNullOrEmpty(newPassword))
                        {
                            //send email with the newly generated password
                            string subject = "Password reset";
                            string body = "Upon your request, your password was reset.<br/>Your new password is:" +
                                            newPassword +
                                            "<br/>If you have not requested a password reset, please contact us.";
                            MbCommon.SendEmail(subject, body, model.Email);
                        }
                    }
                }
            }
            return RedirectToAction("ResetPasswordSuccess");
        }

        public ActionResult ResetPasswordSuccess()
        {
            return View();
        }

        //
        // GET: /Account/LogOff

        public ActionResult Logout()
        {
            FormsAuthentication.SignOut();
            return RedirectToAction("Index", "Default");
        }

        //
        // GET: /Account/Register

        public ActionResult Register()
        {
            return View();
        }

        //
        // POST: /Account/Register

        [HttpPost]
        public ActionResult Register(RegisterModel model)
        {
            if (ModelState.IsValid)
            {
                // Attempt to register the user
                MembershipCreateStatus createStatus;
                Membership.CreateUser(model.Email, model.Password, model.Email, null, null, true, null,
                                        out createStatus);
                if (createStatus == MembershipCreateStatus.Success)
                {
                    //add roles
                    Roles.AddUserToRoles(model.Email, new string[] { "SiteAdmin" });
                }
                
                if (createStatus == MembershipCreateStatus.Success)
                {

                    return Redirect("~/Admin/Account/RegisterSuccess");
                }
                else
                {
                    ModelState.AddModelError("", ErrorCodeToString(createStatus));
                }
            }

            // If we got this far, something failed, redisplay form
            return View(model);
        }
        //
        // GET: /Account/RegisterSuccess
        public ViewResult RegisterSuccess()
        {
            return View();
        }
        //
        // GET: /Account/ChangePassword
        [Authorize]
        public ActionResult ChangePassword()
        {
            return View();
        }

        //
        // POST: /Account/ChangePassword

        [Authorize]
        [HttpPost]
        public ActionResult ChangePassword(ChangePasswordModel model)
        {
            if (ModelState.IsValid)
            {

                // ChangePassword will throw an exception rather
                // than return false in certain failure scenarios.
                bool changePasswordSucceeded;
                try
                {
                    MembershipUser currentUser = Membership.GetUser(User.Identity.Name, true /* userIsOnline */);
                    changePasswordSucceeded = currentUser.ChangePassword(model.OldPassword, model.NewPassword);
                }
                catch (Exception)
                {
                    changePasswordSucceeded = false;
                }

                if (changePasswordSucceeded)
                {
                    return RedirectToAction("ChangePasswordSuccess");
                }
                else
                {
                    ModelState.AddModelError("", "The current password is incorrect or the new password is invalid.");
                }
            }

            // If we got this far, something failed, redisplay form
            return View(model);
        }

        // GET: /Account/ChangePasswordSuccess

        public ActionResult ChangePasswordSuccess()
        {
            return View();
        }

        #region Status Codes
        private static string ErrorCodeToString(MembershipCreateStatus createStatus)
        {
            // See http://go.microsoft.com/fwlink/?LinkID=177550 for
            // a full list of status codes.
            switch (createStatus)
            {
                case MembershipCreateStatus.DuplicateUserName:
                    return "User name already exists. Please enter a different user name.";

                case MembershipCreateStatus.DuplicateEmail:
                    return "A user name for that e-mail address already exists. Please enter a different e-mail address.";

                case MembershipCreateStatus.InvalidPassword:
                    return "The password provided is invalid. Please enter a valid password value.";

                case MembershipCreateStatus.InvalidEmail:
                    return "The e-mail address provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.InvalidAnswer:
                    return "The password retrieval answer provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.InvalidQuestion:
                    return "The password retrieval question provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.InvalidUserName:
                    return "The user name provided is invalid. Please check the value and try again.";

                case MembershipCreateStatus.ProviderError:
                    return "The authentication provider returned an error. Please verify your entry and try again. If the problem persists, please contact your system administrator.";

                case MembershipCreateStatus.UserRejected:
                    return "The user creation request has been canceled. Please verify your entry and try again. If the problem persists, please contact your system administrator.";

                default:
                    return "An unknown error occurred. Please verify your entry and try again. If the problem persists, please contact your system administrator.";
            }
        }
        #endregion
    }
}
