﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;
using mobx.mobi.Areas.Admin.Models;
using mobx.mobi.DAL;
using mobx.mobi.Core;
using System.Security.Permissions;
using mobx.mobi.DAL.Managers;

namespace mobx.mobi.Areas.Admin.Controllers
{
    [PageAttribute(
        ClassName="mobx.mobi.Areas.Admin.Controllers.ContactFormController",
        ShowInGallery = true,
        SortOrder=1,
        ThumbImageUrl = "",
        Title = "Contact Form",
        Description = "Contact form page")]
    [AspNetHostingPermission(SecurityAction.Demand,
        Level = AspNetHostingPermissionLevel.Minimal),
    AspNetHostingPermission(SecurityAction.InheritanceDemand,
        Level = AspNetHostingPermissionLevel.Minimal)]
    public class ContactFormController : Controller, ICmsPage
    {
        /// <summary>
        /// GET: /Admin/ContactForm/Create
        /// </summary>
        /// <returns></returns>
        [Authorize]
        public ActionResult Create()
        {
            ContactFormModel model = new ContactFormModel();
            model.CurrentPage = new MbPage(this);
            model.ShowContactForm = false;
            return View("Edit", model);
        } 

        /// <summary>
        /// POST: /Admin/ContactForm/Create
        /// </summary>
        /// <param name="contactFormModel"></param>
        /// <returns></returns>
         /// <summary>
        ///  GET: /Admin/ContactForm/Edit/5
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [Authorize]
        public ActionResult Edit(int id)
        {
            PageManager pageManager = new PageManager();
            MbPage mbpage = pageManager.GetPage(id);
            ContactFormModel contactForm = new ContactFormModel(mbpage);
            return View(contactForm);
        }
        [Authorize, HttpPost]
        public ActionResult Save(ContactFormModel contactFormModel)
        {
            if (ModelState.IsValid)
            {
                string errorMessage = SavePage(contactFormModel);
                if (string.IsNullOrEmpty(errorMessage))
                {
                    return RedirectToAction("Index", "Default");
                }
                else
                {
                    ModelState.AddModelError("", errorMessage);
                }
            }
            return View("Edit", contactFormModel);
        }
        /// <summary>
        /// POST: /Admin/ContactForm/Delete/
        /// </summary>
        /// <param name="contactFormModel"></param>
        /// <returns></returns>
        [Authorize, HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(ContactFormModel model)
        {
            if (model != null)
            {
                PageManager pageManager = new PageManager();
                string errorMessage = null;
                pageManager.DeletePage(model.CurrentPage.PageId, out errorMessage);
                if (string.IsNullOrEmpty(errorMessage))
                {
                    return RedirectToAction("Index", "Default");
                }
                else
                {
                    ModelState.AddModelError("", errorMessage);
                }
            }
            return View("Edit", model);
        }

        private string SavePage(ContactFormModel contactFormModel)
        {
            string errorMessage;
            if (contactFormModel != null &&
                contactFormModel.ShowContactForm &&
                string.IsNullOrEmpty(contactFormModel.Email))
            {
                errorMessage = "You must input your email in order to use contact form";
                return errorMessage;
            }
            contactFormModel.CurrentPage.PageType = this.ToString();
            ContactFormBase contactBase = contactFormModel.ToBase();
            MbPage mbPage = contactFormModel.CurrentPage;
            mbPage.SetContent(contactBase);

            PageManager pageManager = new PageManager();
            pageManager.UpsertPage(mbPage, out errorMessage);
            return errorMessage;
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
        }
    }
}