﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.IO;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Script.Serialization;
using System.Security.Permissions;
using mobx.mobi.Areas.Admin.Models;
using mobx.mobi.DAL;
using mobx.mobi.Core;
using mobx.mobi.DAL.Managers;

namespace mobx.mobi.Areas.Admin.Controllers
{
    [PageAttribute(
        ClassName = "mobx.mobi.Areas.Admin.Controllers.ImageWithTextController",
        ShowInGallery = true,
        SortOrder=1,
        ThumbImageUrl = "",
        Title = "Image with text",
        Description = "A page containing some text and optionally an image")]
    [AspNetHostingPermission(SecurityAction.Demand,
        Level = AspNetHostingPermissionLevel.Minimal),
    AspNetHostingPermission(SecurityAction.InheritanceDemand,
        Level = AspNetHostingPermissionLevel.Minimal)]
    public class ImageWithTextController : Controller, ICmsPage
    {
        /// <summary>
        /// GET: /Admin/ImageWithText/Create
        /// </summary>
        /// <returns></returns>
        [Authorize]
        public ActionResult Create()
        {
            ImageWithTextModel model = new ImageWithTextModel();
            model.CurrentPage = new MbPage(this);
            return View("Edit", model);
        } 

        /// <summary>
        ///  GET: /Admin/ImageWithText/Edit/5
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [Authorize]
        public ActionResult Edit(int id)
        {
            PageManager pageManager = new PageManager();
            MbPage mbpage = pageManager.GetPage(id);
            ImageWithTextModel model = new ImageWithTextModel(mbpage);
            return View(model);
        }

        /// <summary>
        /// POST: /Admin/ImageWithText/Edit/5 or POST: /Admin/ImageWithText/Create
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [Authorize, HttpPost]
        public ActionResult Save(ImageWithTextModel model, HttpPostedFileBase theFile)
        {
            if (ModelState.IsValid)
            {
                PageManager pageManager = new PageManager();
                bool valid = pageManager.IsPageAliasValid(model.CurrentPage.PageId, model.CurrentPage.Alias);
                if (valid)
                {
                    string errorMessage = null;
                    MbPage outPage = null;
                    errorMessage = SavePage(model, out outPage);

                    if (string.IsNullOrEmpty(errorMessage) &&
                        outPage != null &&
                        theFile != null && 
                        theFile.ContentLength > 0)
                    {
                        //if no error so far then continue to save page
                        string fileName = Path.GetFileName(theFile.FileName);
                        string url = "~/Uploads/page" + outPage.PageId.ToString() + "/";
                        string folder = Server.MapPath(url);
                        url = url + fileName;
                        string path = Server.MapPath(url);
                        if (!Directory.Exists(folder))
                        {
                            // Create the subfolder
                            Directory.CreateDirectory(folder);
                        }

                        if (System.IO.File.Exists(path))
                        {
                            errorMessage = "A file with the same name already exists";
                        }
                        else
                        {
                            theFile.SaveAs(path);
                            model.ImageUrl = url;
                            //save the page again
                            errorMessage = SavePage(model, out outPage);
                        }
                    }                    
                  

                    if (string.IsNullOrEmpty(errorMessage))
                    {
                        return RedirectToAction("Index", "Default");
                    }
                    else
                    {
                        ModelState.AddModelError("", errorMessage);
                    }
                }
                else
                {
                    ModelState.AddModelError("", "A page with the same URL Name already exists");
                }
            }
            return View("Edit", model);
        }

        /// <summary>
        /// POST: /Admin/ImageWithText/Delete/
        /// </summary>
        /// <param name="model"></param>
        /// <returns></returns>
        [Authorize, HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(ImageWithTextModel model)
        {
            if (model != null)
            {
                PageManager pageManager = new PageManager();
                string errorMessage = null;
                pageManager.DeletePage(model.CurrentPage.PageId, out errorMessage);
                if (string.IsNullOrEmpty(errorMessage))
                {
                    return RedirectToAction("Index", "Default");
                }
                else
                {
                    ModelState.AddModelError("", errorMessage);
                }
            }
            return View("Edit", model);
        }

        private string SavePage(ImageWithTextModel model, out MbPage outPage)
        {
            model.CurrentPage.PageType = this.ToString();

            ImageWithTextBase baseModel = model.ToBase();
            MbPage mbPage = model.CurrentPage;
            mbPage.SetContent(baseModel);

            PageManager pageManager = new PageManager();
            string errorMessage;
            outPage = pageManager.UpsertPage(mbPage, out errorMessage);
            //MbCmsCommon.RegisterCmsRoutes();
            return errorMessage;
        }

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
        }
    }
}