﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using mobx.mobi.DAL;

namespace mobx.mobi
{
    // Note: For instructions on enabling IIS6 or IIS7 classic mode, 
    // visit http://go.microsoft.com/?LinkId=9394801

    public class MvcApplication : System.Web.HttpApplication
    {
        public class CustomControllerFactory : DefaultControllerFactory
        {
            public override IController CreateController(RequestContext requestContext, string controller)
            {
                if (requestContext.RouteData.DataTokens["area"] == null ||
                    (requestContext.RouteData.DataTokens["area"] != null && requestContext.RouteData.DataTokens["area"].ToString() == string.Empty))
                {
                    // collect route data
                    //string controller = requestContext.RouteData.Values["controller"].ToString();
                    string action = requestContext.RouteData.Values["action"].ToString();
                    //string id = requestContext.RouteData.Values["id"].ToString();
                    string pageAlias = string.Empty;
           
                    MbPage page = null;
                    if (controller.ToLower() == "default" &&
                        action.ToLower() == "index")
                    {
                        //show default page                       
                        controller = "Default";
                        action = "DefaultPage";
                        pageAlias = "Default";
                    } 
                    else if (controller.ToLower() == "install" &&
                             (action.ToLower() == "index" || action.ToLower() == "success"))
                    {
                        //controller = "Default";
                        //action = "DefaultPage";
                        pageAlias = "Install";
                    }
                    else
                    {
                        //show specific page
                        string preview = requestContext.HttpContext.Request.QueryString["preview"];
                        if (!string.IsNullOrEmpty(preview) &&
                            preview == "1")
                        {
                        	page = Singleton.DB.MbPages              
                                    .Where(p => p.Alias.ToLower() == controller.ToLower())
                                    .FirstOrDefault();
						}
						else
						{
							//only show published
                        	page = Singleton.DB.MbPages
                                    .Where(p => p.Published)
                                    .Where(p => p.Alias.ToLower() == controller.ToLower())
                                    .FirstOrDefault();
						}
                        if (page != null)
                        {
                            controller = MbCommon.ParseControllerName(page.PageType);
                            pageAlias = page.Alias;
                        }
                        else
                        {
                            //site exists, but no pages in site
                            ////controller = "Default";
                            ////action = "EmptyPage";
                        }
                    }
                    

                    requestContext.RouteData.Values["controller"] = controller;
                    requestContext.RouteData.Values["action"] = action;
                    requestContext.RouteData.Values["pagealias"] = pageAlias;
                }
                return base.CreateController(requestContext, controller);
            }
        }

        public static void RegisterGlobalFilters(GlobalFilterCollection filters)
        {
            filters.Add(new HandleErrorAttribute());
        }
        public static void RegisterRoutes(RouteCollection routes)
        {
            routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

            routes.MapRoute(
                "Default", // Route name
                "{controller}/{action}/{id}", // URL with parameters
                new { controller = "Default", action = "Index", id = UrlParameter.Optional }, // Parameter defaults
                new string[] { "mobx.mobi.Controllers" }
            );
        }
        protected void Application_Start()
        {
            AreaRegistration.RegisterAllAreas();

            ControllerBuilder.Current.SetControllerFactory(new CustomControllerFactory());
            RegisterRoutes(RouteTable.Routes); // this already exists by default

        }
    
    }
}