﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Configuration;
using System.Web.Routing;
using mobx.mobi.DAL;
using mobx.mobi.DAL.Managers;
using System.Linq.Expressions;

namespace mobx.mobi
{
    public static class HtmlHelperExtensions
    {
        public static string GetSelected(this HtmlHelper helper, string val1, string val2)
        {
            return (val1 == val2) ? " selected=\"selected\"" : string.Empty;
        }
        public static string CurrentTheme(this HtmlHelper helper)
        {
            SettingManager settingMan = new SettingManager();
            string theme = settingMan.GetTheme();
            return theme;
        }
        public static string CurrentGoogleAnalytics(this HtmlHelper helper)
        {
            SettingManager settingMan = new SettingManager();
            string theme = settingMan.GetGoogleAnalytics();
            return theme;
        }        
        public static string CurrentTransition(this HtmlHelper helper)
        {
            SettingManager settingMan = new SettingManager();
            string theme = settingMan.GetTransition();
            return theme;
        }
        public static bool IsCurrentControllerAndAction(this HtmlHelper helper, string controllerName, string actionName)
        {
            string normalizedControllerName = controllerName.EndsWith("Controller") ? controllerName : String.Format("{0}Controller", controllerName);

            if (helper.ViewContext == null) return false;
            if (String.IsNullOrEmpty(actionName)) return false;

            if (helper.ViewContext.Controller.GetType().Name.Equals(normalizedControllerName, StringComparison.InvariantCultureIgnoreCase) &&
                helper.ViewContext.Controller.ValueProvider.GetValue("action").AttemptedValue.Equals(actionName, StringComparison.InvariantCultureIgnoreCase))
            {
                return true;
            }
            return false;
        }
        public static bool IsCurrentPath(this HtmlHelper helper, string path)
        {
            if (helper != null &&
                    helper.ViewContext != null &&
                    helper.ViewContext.RequestContext != null &&
                    helper.ViewContext.RequestContext.HttpContext != null &&
                    helper.ViewContext.RequestContext.HttpContext.Request != null &&
                    !string.IsNullOrEmpty(helper.ViewContext.RequestContext.HttpContext.Request.Path) &&
                    !string.IsNullOrEmpty(path))
            {
                string currentPath = helper.ViewContext.RequestContext.HttpContext.Request.Path;
                if (!currentPath.EndsWith("/"))
                {
                    currentPath += "/";
                }

                if (!path.EndsWith("/"))
                {
                    path += "/";
                }

                if (currentPath.ToLower() == path.ToLower())
                {
                    return true;
                }
                else if (helper.ViewContext.RequestContext.RouteData.GetRouteAlias() == "Default")
                {
                    PageManager pageManager = new PageManager();
                    MbPage defaultPage = pageManager.GetDefaultPage();
                    string defaultPageAlias = defaultPage.Alias;
                    if (!defaultPageAlias.StartsWith("/"))
                    {
                        defaultPageAlias = "/" + defaultPageAlias;
                    }
                    if (!defaultPageAlias.EndsWith("/"))
                    {
                        defaultPageAlias += "/";
                    }
                    if (defaultPageAlias.ToLower() == currentPath.ToLower() ||
                        (currentPath.ToLower() == "/" && defaultPageAlias.ToLower() == path.ToLower()))
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        //http://haacked.com/archive/2010/11/28/getting-the-route-name-for-a-route.aspx
        public static string GetRouteAlias(this Route route)
        {
            if (route == null)
            {
                return null;
            }
            return route.DataTokens.GetRouteAlias();
        }

        public static string GetRouteAlias(this RouteData routeData)
        {
            if (routeData == null)
            {
                return null;
            }
            return routeData.DataTokens.GetRouteAlias();
        }

        public static string GetRouteAlias(this RouteValueDictionary routeValues)
        {
            if (routeValues == null)
            {
                return null;
            }
            object routeAlias = null;
            routeValues.TryGetValue("__RouteAlias", out routeAlias);
            return routeAlias as string;
        }

        public static Route SetRouteAlias(this Route route, string routeAlias)
        {
            if (route == null)
            {
                throw new ArgumentNullException("route");
            }
            if (route.DataTokens == null)
            {
                route.DataTokens = new RouteValueDictionary();
            }
            route.DataTokens["__RouteAlias"] = routeAlias;
            return route;
        }

        public static MvcHtmlString GetDisplayName<TModel, TProperty>(this HtmlHelper<TModel> htmlHelper, Expression<Func<TModel, TProperty>> expression)
        {
            var metaData = ModelMetadata.FromLambdaExpression<TModel, TProperty>(expression, htmlHelper.ViewData);
            string value = metaData.DisplayName ?? (metaData.PropertyName ?? ExpressionHelper.GetExpressionText(expression));
            return MvcHtmlString.Create(value);
        }
    }
}